-- complain if script is sourced in psql, rather than via CREATE EXTENSION
\echo Use "CREATE EXTENSION MODULE_PATHNAME" to load this file. \quit

/* ***********************************************
 * Hijri Calendar functions for PostgreSQL
 * *********************************************** */

---- in/out functions
--hijri

CREATE TYPE hijri;

CREATE FUNCTION hijri_in(
	cstring)
    RETURNS hijri
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE 
    
AS 'pgHijri', 'hijri_in';

CREATE FUNCTION hijri_out(
	date)
    RETURNS cstring
    LANGUAGE 'c'
    COST 1
    STABLE STRICT PARALLEL SAFE
  
AS 'pgHijri', 'hijri_out';

CREATE FUNCTION hijri_out(
	hijri)
    RETURNS cstring
    LANGUAGE 'c'
    COST 1
    STABLE STRICT PARALLEL SAFE
  
AS 'pgHijri', 'hijri_out';

CREATE TYPE hijri (
  INPUT          = hijri_in,
  OUTPUT         = hijri_out,
  LIKE           = date
);

CREATE CAST (date as hijri) WITHOUT FUNCTION AS IMPLICIT;
CREATE CAST (hijri as date) WITHOUT FUNCTION AS IMPLICIT;

--htimestamp
CREATE TYPE htimestamp;
CREATE FUNCTION htimestamp_in(
	cstring,
	oid,
	integer)
    RETURNS htimestamp
    LANGUAGE 'c'
	COST 1
    STABLE STRICT PARALLEL SAFE
	AS 'pgHijri', 'htimestamp_in';	
	
	CREATE FUNCTION htimestamp_out(
	htimestamp)
    RETURNS cstring
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
	AS 'pgHijri', 'htimestamp_out';
	
	CREATE TYPE htimestamp (
  INPUT          = htimestamp_in,
  OUTPUT         = htimestamp_out,
  LIKE           = timestamp
);

CREATE CAST (timestamp as htimestamp) WITHOUT FUNCTION AS IMPLICIT;
CREATE CAST (htimestamp as timestamp) WITHOUT FUNCTION AS IMPLICIT;

--htimestamptz
CREATE TYPE htimestamptz;
CREATE FUNCTION htimestamptz_in(
	cstring,
	oid,
	integer)
    RETURNS htimestamptz
    LANGUAGE 'c'
	COST 1
    STABLE STRICT PARALLEL SAFE
	AS 'pgHijri', 'htimestamptz_in';	
	
	CREATE FUNCTION htimestamptz_out(
	htimestamptz)
    RETURNS cstring
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
	AS 'pgHijri', 'htimestamptz_out';
	
	CREATE TYPE htimestamptz (
  INPUT          = htimestamptz_in,
  OUTPUT         = htimestamptz_out,
  LIKE           = timestamptz
);

CREATE CAST (timestamptz as htimestamptz) WITHOUT FUNCTION AS IMPLICIT;
CREATE CAST (htimestamptz as timestamptz) WITHOUT FUNCTION AS IMPLICIT;

----casting functions
--htimestamp
CREATE FUNCTION "htimestamp"(
	date)
    RETURNS htimestamp
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamp$BODY$;

CREATE CAST (date AS htimestamp) WITH FUNCTION htimestamp(date) AS IMPLICIT;

CREATE FUNCTION "htimestamp"(
	hijri)
    RETURNS htimestamp
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamp$BODY$;

CREATE CAST (hijri AS htimestamp) WITH FUNCTION htimestamp(hijri) AS IMPLICIT;

CREATE FUNCTION "htimestamp"(
	timestamp with time zone)
    RETURNS htimestamp
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_timestamp$BODY$;

CREATE CAST (timestamptz AS htimestamp) WITH FUNCTION htimestamp(timestamptz) AS IMPLICIT;

CREATE FUNCTION "htimestamp"(
	htimestamptz)
    RETURNS htimestamp
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_timestamp$BODY$;

CREATE CAST (htimestamptz AS htimestamp) WITH FUNCTION htimestamp(htimestamptz) AS IMPLICIT;

--timestamp

CREATE FUNCTION "timestamp"(
	hijri)
    RETURNS timestamp
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamp$BODY$;

CREATE CAST (hijri AS timestamp) WITH FUNCTION "timestamp"(hijri) AS IMPLICIT;

CREATE FUNCTION "timestamp"(
	htimestamptz)
    RETURNS timestamp
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_timestamp$BODY$;

CREATE CAST (htimestamptz AS timestamp) WITH FUNCTION "timestamp"(htimestamptz) AS IMPLICIT;


--timestamptz

CREATE FUNCTION timestamptz(
	hijri)
    RETURNS timestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamptz$BODY$;

CREATE CAST (hijri AS timestamptz) WITH FUNCTION timestamptz(hijri) AS IMPLICIT;


CREATE FUNCTION timestamptz(
	htimestamp)
    RETURNS timestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_timestamptz$BODY$;

CREATE CAST (htimestamp AS timestamptz) WITH FUNCTION timestamptz(htimestamp) AS IMPLICIT;

--htimestamptz

CREATE FUNCTION htimestamptz(
	date)
    RETURNS htimestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamptz$BODY$;

CREATE CAST (date AS htimestamptz) WITH FUNCTION htimestamptz(date) AS IMPLICIT;

CREATE FUNCTION htimestamptz(
	hijri)
    RETURNS htimestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$date_timestamptz$BODY$;

CREATE CAST (hijri AS htimestamptz) WITH FUNCTION htimestamptz(hijri) AS IMPLICIT;

CREATE FUNCTION htimestamptz(
	timestamp without time zone)
    RETURNS htimestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_timestamptz$BODY$;

CREATE CAST (timestamp AS htimestamptz) WITH FUNCTION htimestamptz(timestamp) AS IMPLICIT;

CREATE FUNCTION htimestamptz(
	htimestamp)
    RETURNS htimestamptz
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_timestamptz$BODY$;

CREATE CAST (htimestamp AS htimestamptz) WITH FUNCTION htimestamptz(htimestamp) AS IMPLICIT;

--hijri

CREATE FUNCTION hijri(
	timestamp with time zone)
    RETURNS hijri
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_date$BODY$;

CREATE CAST (timestamptz AS hijri) WITH FUNCTION hijri(timestamptz) AS IMPLICIT;

CREATE FUNCTION hijri(
	htimestamptz)
    RETURNS hijri
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_date$BODY$;

CREATE CAST (htimestamptz AS hijri) WITH FUNCTION hijri(htimestamptz) AS IMPLICIT;

CREATE FUNCTION hijri(
	timestamp without time zone)
    RETURNS hijri
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_date$BODY$;

CREATE CAST (timestamp AS hijri) WITH FUNCTION hijri(timestamp) AS IMPLICIT;

CREATE FUNCTION hijri(
	htimestamp)
    RETURNS hijri
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_date$BODY$;

CREATE CAST (htimestamp AS hijri) WITH FUNCTION hijri(htimestamp) AS IMPLICIT;

--date

CREATE FUNCTION date(
	htimestamptz)
    RETURNS date
    LANGUAGE 'internal'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamptz_date$BODY$;

CREATE CAST (htimestamptz AS date) WITH FUNCTION date(htimestamptz) AS IMPLICIT;

CREATE FUNCTION date(
	htimestamp)
    RETURNS date
    LANGUAGE 'internal'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$timestamp_date$BODY$;

CREATE CAST (htimestamp AS date) WITH FUNCTION date(htimestamp) AS IMPLICIT;

----part functions

--part
CREATE FUNCTION hijri_part(
	text,
	timestamp with time zone)
    RETURNS double precision
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE 
    
AS 'MODULE_PATHNAME', 'htimestamptz_part';
 
  
CREATE FUNCTION hijri_part(
	text,
	timestamp without time zone)
    RETURNS double precision
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE 
    
AS 'MODULE_PATHNAME', 'htimestamp_part';


CREATE FUNCTION hijri_part(
	text,
	date)
    RETURNS double precision
    LANGUAGE 'sql'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$select hijri_part($1, cast($2 as timestamp without time zone))$BODY$;


CREATE FUNCTION date_part(
	text,
	htimestamp)
    RETURNS double precision
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE 
    
AS 'MODULE_PATHNAME', 'htimestamp_part';

CREATE FUNCTION date_part(
	text,
	hijri)
    RETURNS double precision
    LANGUAGE 'sql'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS $BODY$select hijri_part($1, cast($2 as timestamp without time zone))$BODY$;

CREATE FUNCTION date_part(
	text,
	htimestamptz)
    RETURNS double precision
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE 
    
AS 'MODULE_PATHNAME', 'htimestamptz_part';

----make functions
CREATE FUNCTION make_htimestamp(
	year integer,
	month integer,
	mday integer,
	hour integer,
	min integer,
	sec double precision)
    RETURNS htimestamp
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'make_htimestamp';

CREATE FUNCTION make_hijri(
	year integer,
	month integer,
	day integer)
    RETURNS hijri
    LANGUAGE 'c'

    COST 1
    IMMUTABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'make_hijri';

CREATE FUNCTION make_htimestamptz(
	year integer,
	month integer,
	mday integer,
	hour integer,
	min integer,
	sec double precision)
    RETURNS htimestamptz
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'make_htimestamptz';

CREATE FUNCTION make_htimestamptz(
	year integer,
	month integer,
	mday integer,
	hour integer,
	min integer,
	sec double precision,
	timezone text)
    RETURNS htimestamptz
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'make_htimestamptz_at_timezone';

---- formatting
--to_char

CREATE FUNCTION h_to_char(
	timestamp without time zone,
	text)
    RETURNS text
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'htimestamp_to_char';

CREATE FUNCTION to_char(
	htimestamp,
	text)
    RETURNS text
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'htimestamp_to_char';

CREATE FUNCTION h_to_char(
	timestamptz,
	text)
    RETURNS text
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'htimestamptz_to_char';

CREATE FUNCTION to_char(
	htimestamptz,
	text)
    RETURNS text
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'htimestamptz_to_char';

CREATE FUNCTION to_htimestamp(
	text,
	text)
    RETURNS htimestamp
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'to_htimestamp';

CREATE FUNCTION to_hijri(
	text,
	text)
    RETURNS hijri
    LANGUAGE 'c'

    COST 1
    STABLE STRICT PARALLEL SAFE
    
AS 'MODULE_PATHNAME', 'to_hijri';